@@ -24,7 +24,7 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.7 Sun, 09 Jul 2000 14:46:03 +1000 kaos $"
+#ident "$Id: obj.h 1.8 Tue, 25 Jul 2000 16:31:08 +1000 kaos $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
@@ -113,7 +113,8 @@ enum obj_reloc
   obj_reloc_ok,
   obj_reloc_overflow,
   obj_reloc_dangerous,
-  obj_reloc_unhandled
+  obj_reloc_unhandled,
+  obj_reloc_constant_gp
 };
 
 struct obj_string_patch_struct
