@@ -48,7 +48,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: insmod.c 1.20 Sun, 09 Jul 2000 14:48:27 +1000 kaos $"
+#ident "$Id: insmod.c 1.21 Sun, 09 Jul 2000 22:30:18 +1000 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -1338,6 +1338,10 @@ int INSMOD_MAIN(int argc, char **argv)
 		else if (len > 4 && p[len - 4] == '.' && p[len - 3] == 'm'
 			 && p[len - 2] == 'o' && p[len - 1] == 'd')
 			len -= 4;
+#ifdef CONFIG_USE_ZLIB
+		else if (len > 5 && !strcmp(p + len - 5, ".o.gz"))
+			len -= 5;
+#endif
 
 		m_name = xmalloc(len + 1);
 		memcpy(m_name, p, len);
