@@ -66,6 +66,7 @@ typedef struct DESC {
 	NODE *nod;
 	LINK *options;
 	short int mode;
+	short int autoclean;
 } DESC;
 static char *modename[] = { "NORMAL", "ABOVE", "BELOW" };
 
@@ -107,18 +108,27 @@ static void verbose(const char *ctl,...)
 static const char *stripo(const char *fname)
 {
 	const char *pt = fname;
+	char *ptv;
+	int len;
 
 	if ((pt = strrchr(fname, '/')) != NULL)
 		fname = pt + 1;
-	if ((pt = strrchr(fname, '.')) &&
-	    ((strcmp(pt, ".o") == 0) || (strcmp(pt, ".mod") == 0))) {
-		char *leak = (char *)xmalloc(pt - fname + 1);
+	len = strlen(fname);
+	if (len > 2 && !strcmp(fname + len - 2, ".o"))
+		len -= 2;
+	else if (len > 4 && !strcmp(fname + len - 4, ".mod"))
+		len -= 4;
+#ifdef CONFIG_USE_ZLIB
+	else if (len > 5 && !strcmp(fname + len - 5, ".o.gz"))
+		len -= 5;
+#endif
+	else return fname;
 
-		strncpy(leak, fname, pt - fname);
-		leak[pt - fname] = '\0';
-		fname = leak;
-	}
-	return fname;
+	ptv = xmalloc(len + 1);
+
+	strncpy(ptv, fname, len);
+	ptv[len] = '\0';
+	return ptv;	/* memleak */
 }
 
 /*
@@ -155,14 +165,14 @@ static GLOB_LIST *any_probeall_list(const char *name)
  *	Return the options associated with a module.
  *	Return NULL if there is none.
  */
-static GLOB_LIST *any_options(const char *mod)
+static OPT_LIST *any_options(const char *mod)
 {
 	const char *modname = stripo(mod);
 	int i;
 
 	for (i = 0; opt_list && opt_list[i].name; ++i) {
 		if (strcmp(opt_list[i].name, modname) == 0)
-			return opt_list[i].opts;
+			return opt_list+i;
 	}
 	return NULL;
 }
@@ -293,6 +303,7 @@ static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 {
 	DESC *desc;
 	GLOB_LIST *g;
+	OPT_LIST *o;
 	NODE *nod = NULL;
 	const char *p;
 	int aliases = 0, ret = 0;
@@ -305,6 +316,7 @@ static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 	desc->options = NULL;
 	desc->kname = NULL;
 	desc->nod = objname ? nod : NULL;
+	desc->autoclean = 1;
 
 	for (p = name; p; p = allow_alias ? any_alias(p) : NULL) {
 		if (strcmp(p, "off") == 0) {
@@ -313,8 +325,12 @@ static DESC *build_desc(const char *name, const char *objname, int allow_alias)
 		}
 		desc->objkey = p;
 		desc->name = linkit(LAST, (void *)p, desc->name);
+		if (!(o = any_options(p)))
+			continue;
 
-		if ((g = any_options(p))) {
+		if (!o->autoclean) desc->autoclean = 0;
+
+		if ((g = o->opts)) {
 			desc->options = linkit(FIRST, g, desc->options);
 			if (!desc->kname) {
 				int i;
@@ -538,9 +554,16 @@ static GLOB_LIST *config_locate(const char *match, const char *type)
 	sprintf(match_o, "%s.o", match);
 	g = config_lstmod(match_o, type, 0);
 	if (g == NULL || g->pathc == 0) {
-		g = config_lstmod((char *)match, type, 0);
-		if (g == NULL || g->pathc == 0)
-			return NULL;
+#ifdef CONFIG_USE_ZLIB
+		sprintf(match_o, "%s.o.gz", match);
+		g = config_lstmod(match_o, type, 0);
+		if (g == NULL || g->pathc == 0) 
+#endif
+		{
+			g = config_lstmod((char *)match, type, 0);
+			if (g == NULL || g->pathc == 0)
+				return NULL;
+		}
 	}
 	/* else */
 
@@ -1002,7 +1025,7 @@ static int insmod(DESC *desc, LINK **new_in_kernel, char *options[])
 
 		/* Locking is _always_ done in insmod now: -L is redundant */
 		my_argv[my_argc++] = "/sbin/insmod";
-		if (flag_autoclean)
+		if (flag_autoclean && desc->autoclean)
 			my_argv[my_argc++] = "-k";
 
 		if (quiet)
@@ -1122,8 +1145,11 @@ static int insmod(DESC *desc, LINK **new_in_kernel, char *options[])
 		}
 		if (runit) {
 			int insmod_main(int argc, char **argv);
+			int flag_autoclean_save = flag_autoclean;
 
+			flag_autoclean = 0;
 			ret = insmod_main(my_argc, my_argv);
+			flag_autoclean = flag_autoclean_save;
 		}
 #else
 		if (runit || verbose) {
