@@ -109,10 +109,19 @@ int quick = 0;			/* Option -A */
 static time_t dep_mtime;
 static int check_update (const char *file, const struct stat *sb, int flag)
 {
-	const char *p;
-	p = strrchr (file, '.');
-	if (p && !strcmp (p, ".o") && sb->st_mtime > dep_mtime)
+	int len = strlen(file);
+
+	if (sb->st_mtime <= dep_mtime)
+		return 0;
+
+	if (len > 2 && !strcmp(file + len - 2, ".o"))
 		return 1;
+	else if (len > 4 && !strcmp(file + len - 4, ".mod"))
+		return 1;
+#ifdef CONFIG_USE_ZLIB
+	else if (len > 5 && !strcmp(file + len - 5, ".o.gz"))
+		return 1;
+#endif
 	return 0;
 }
 
@@ -270,14 +279,22 @@ static GLOB_LIST *addlist(GLOB_LIST *orig, GLOB_LIST *add)
 }
 
 static void decode_list(int *n, OPT_LIST **list, char *arg, int adding,
-			char *version)
+			char *version, int opts)
 {
 	GLOB_LIST *pg;
 	GLOB_LIST *prevlist = NULL;
-	int i;
+	int i, autoclean = 1;
 	int where = *n;
 	char *arg2 = next_word(arg);
 
+	if (opts && !strcmp (arg, "-k")) {
+		if (!*arg2)
+			error("Missing module argument after -k\n");
+		arg = arg2;
+		arg2 = next_word(arg);
+		autoclean = 0;
+	}
+
 	for (i = 0; i < *n; ++i) {
 		if (strcmp((*list)[i].name, arg) == 0) {
 			if (adding)
@@ -293,9 +310,11 @@ static void decode_list(int *n, OPT_LIST **list, char *arg, int adding,
 		(*list) = (OPT_LIST *)xrealloc((*list),
 			  (*n + 2) * sizeof(OPT_LIST));
 		(*list)[*n].name = xstrdup(arg);
+		(*list)[*n].autoclean = autoclean;
 		*n += 1;
 		memset(&(*list)[*n], 0, sizeof(OPT_LIST));
-	}
+	} else if (!autoclean)
+		(*list)[where].autoclean = 0;
 	pg = (GLOB_LIST *)xmalloc(sizeof(GLOB_LIST));
 	meta_expand(arg2, pg, NULL, version);
 	(*list)[where].opts = addlist(prevlist, pg);
@@ -316,7 +335,7 @@ static void decode_exec(char *arg, int type)
 	}
 }
 
-static int build_list(char **in, OPT_LIST **out, char *version)
+static int build_list(char **in, OPT_LIST **out, char *version, int opts)
 {
 	GLOB_LIST *pg;
 	int i;
@@ -324,11 +343,18 @@ static int build_list(char **in, OPT_LIST **out, char *version)
 	for (i = 0; in[i]; ++i) {
 		char *p = xstrdup(in[i]);
 		char *pt = next_word(p);
+		char *pn = p;
 
 		*out = (OPT_LIST *)xrealloc(*out, (i + 2) * sizeof(OPT_LIST));
+		(*out)[i].autoclean = 1;
+		if (opts && !strcmp (p, "-k")) {
+		    pn = pt;
+		    pt = next_word(pn);
+		    (*out)[i].autoclean = 0;
+		}
 		pg = (GLOB_LIST *)xmalloc(sizeof(GLOB_LIST));
 		meta_expand(pt, pg, NULL, version);
-		(*out)[i].name = xstrdup(p);
+		(*out)[i].name = xstrdup(pn);
 		(*out)[i].opts = pg;
 		free(p);
 	}
@@ -518,25 +544,25 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		 * Build predef options
 		 */
 		if (all && optlist[0])
-			n_opt_list = build_list(optlist, &opt_list, version);
+			n_opt_list = build_list(optlist, &opt_list, version, 1);
 
 		/*
 		 * Build predef above
 		 */
 		if (all && above[0])
-			n_abovelist = build_list(above, &abovelist, version);
+			n_abovelist = build_list(above, &abovelist, version, 0);
 
 		/*
 		 * Build predef below
 		 */
 		if (all && below[0])
-			n_belowlist = build_list(below, &belowlist, version);
+			n_belowlist = build_list(below, &belowlist, version, 0);
 
 		/*
 		 * Build predef aliases
 		 */
 		if (all && aliaslist[0])
-			n_aliases = build_list(aliaslist, &aliases, version);
+			n_aliases = build_list(aliaslist, &aliases, version, 0);
 
 		/* Order and priority is now: (MODPATH + modules.conf) || (predefs + modules.conf) */
 		if ((envpath = getenv("MODPATH")) != NULL) {
@@ -938,7 +964,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		 * above
 		 */
 		else if (all && !assgn && strcmp(parm, "above") == 0) {
-			decode_list(&n_abovelist, &abovelist, arg, adding, version);
+			decode_list(&n_abovelist, &abovelist, arg, adding, version, 0);
 			one_err = 0;
 		}
 
@@ -946,7 +972,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		 * below
 		 */
 		else if (all && !assgn && strcmp(parm, "below") == 0) {
-			decode_list(&n_belowlist, &belowlist, arg, adding, version);
+			decode_list(&n_belowlist, &belowlist, arg, adding, version, 0);
 			one_err = 0;
 		}
 
@@ -954,7 +980,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		 * probe
 		 */
 		else if (all && !assgn && strcmp(parm, "probe") == 0) {
-			decode_list(&n_probe_list, &probe_list, arg, adding, version);
+			decode_list(&n_probe_list, &probe_list, arg, adding, version, 0);
 			one_err = 0;
 		}
 
@@ -962,7 +988,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		 * probeall
 		 */
 		else if (all && !assgn && strcmp(parm, "probeall") == 0) {
-			decode_list(&n_probeall_list, &probeall_list, arg, adding, version);
+			decode_list(&n_probeall_list, &probeall_list, arg, adding, version, 0);
 			one_err = 0;
 		}
 
@@ -970,7 +996,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		 * options
 		 */
 		else if (all && !assgn && strcmp(parm, "options") == 0) {
-			decode_list(&n_opt_list, &opt_list, arg, adding, version);
+			decode_list(&n_opt_list, &opt_list, arg, adding, version, 1);
 			one_err = 0;
 		}
 
@@ -982,7 +1008,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			 * Replace any previous (default) definitions
 			 * for the same module
 			 */
-			decode_list(&n_aliases, &aliases, arg, 0, version);
+			decode_list(&n_aliases, &aliases, arg, 0, version, 0);
 			one_err = 0;
 		}
 
